$ontext
Distillation Columm Optimization with extrinsic function
This example is an application of the PengRobinson.dll and RaoultLaw.dll libraries proposed by
Ph.D. Juan I. Manassaldi (jmanassaldi@frro.utn.edu.ar); Ph.D. Miguel C. Mussati; Ph.D. Nicols J. Scenna; Ph.D. Sergio F. Mussati (mussati@santafe-conicet.gov.ar)
Some features of the libraries:
         Contain a database of 430 pure compounds.
         In a txt file, the IDs of the intervening compounds and their interaction parameters (if necessary) should be defined.
         The input arguments of the function vary with the number of compounds involved.
         They support up to 18 compounds. Temperature + Pressure + 18 compounds = 20 argument (maximum arguments of extrinsic function for GAMS).
         All the functions have as input arguments the Temperature, Pressure and molar fraction of each component of the mixture. For example, for a binary mixture, the functions will have 4 input arguments.
         All the extrinsic functions have an analytic implementation of their gradient vector and Hessian matrix.
         Functions implemented in each library:
                 Liquid and vapor phase density.
                 Liquid and vapor phase enthalpy.
                 Liquid and vapor phase entropy.
         Fugacity of each component in each phase.
         The database for pure compounds corresponds has been taken from:
                 ChemSep v7.15 pure component data - Copyright (c) Harry Kooijman and Ross Taylor (2016) - http://www.perlfoundation.org/artistic_license_2_0
         The libraries were developed with Dev C ++ using tdm-gcc as a compiler.
         More information about the library can be found in the compilation section of the lst file.
$offtext

*Peng Robinson**************************************************************************************
*Compound ID****************************************************************************************
$onecho > PengRobinsonID.txt
ID1 3
ID2 4
ID3 5
ID4 8
ID5 7
$offecho
*Interaction parameter******************************************************************************
$onecho > PengRobinsonaij.txt
a12 -0.0078
a13  0.0033
a14  0.0111
a15  0.0267
a23 -0.0004
a24  0.0005043
a25  0.00067951
a34  0.00021669
a35  0.0174
a45  0.06
$offecho

$funclibin PengRobinson PengRobinson.dll
function h_liq   /PengRobinson.h_liq/;
function f1_liq  /PengRobinson.f1_liq/;
function f2_liq  /PengRobinson.f2_liq/;
function f3_liq  /PengRobinson.f3_liq/;
function f4_liq  /PengRobinson.f4_liq/;
function f5_liq  /PengRobinson.f5_liq/;
function h_vap   /PengRobinson.h_vap/;
function f1_vap  /PengRobinson.f1_vap/;
function f2_vap  /PengRobinson.f2_vap/;
function f3_vap  /PengRobinson.f3_vap/;
function f4_vap  /PengRobinson.f4_vap/;
function f5_vap  /PengRobinson.f5_vap/;
****************************************************************************************************

*Raoult Law*****************************************************************************************
*Compound ID (no interaction parameters are needed)*************************************************
$onecho > RaoultLawID.txt
ID1 3
ID2 4
ID3 5
ID4 8
ID5 7
$offecho

$funclibin RaoultLaw RaoultLaw.dll
function hr_liq   /RaoultLaw.h_liq /;
function f1r_liq  /RaoultLaw.f1_liq/;
function f2r_liq  /RaoultLaw.f2_liq/;
function f3r_liq  /RaoultLaw.f3_liq/;
function f4r_liq  /RaoultLaw.f4_liq/;
function f5r_liq  /RaoultLaw.f5_liq/;
function hr_vap   /RaoultLaw.h_vap/;
function f1r_vap  /RaoultLaw.f1_vap/;
function f2r_vap  /RaoultLaw.f2_vap/;
function f3r_vap  /RaoultLaw.f3_vap/;
function f4r_vap  /RaoultLaw.f4_vap/;
function f5r_vap  /RaoultLaw.f5_vap/;
****************************************************************************************************

sets
i            compounds           /C3,iC4,nC4,iC5,nC5/
s            equilibrium stages  /s1*s11/
condenser(s) condenser stage     /s1/
reboiler(s)  reboiler stage      /s11/
trays(s)     intermediates stage /s2*s10/
;

parameter
P Pressure [bar] /8.27/

xf(i) feed stream mole fraction
/C3 0.05
iC4 0.15
nC4 0.25
iC5 0.2
nC5 0.35/

Pf  feed stream pressue  [bar]                                                  /8.27/
Tf  feed stream pressue  [K] (saturated temperature obtained solving buble.gms  /354.042810726611/
Hlf feed stream enthalpy [kJ(gmol-1)]
LFF  feed stream molar flow [gmol(sec-1)]                                        /12.6/
LKB maximum mole fraction of ligth key component in the bottom                  /0.066271/
HKT maximum mole fraction of heavy key component in the top                     /0.094401/
;
Hlf = 1e-3*h_liq(Tf,Pf,xf('C3'),xf('iC4'),xf('nC4'),xf('iC5'),xf('nC5'));

equation
****************************************************************************************************
*Collum MESH equation*******************************************************************************
eq1,eq2,eq3,eq4,eq5,eq6,eq7,eq8,eq9,eq10
*Peng Robinson Properties packages******************************************************************
eq11_C3_PR,eq11_iC4_PR,eq11_nC4_PR,eq11_iC5_PR,eq11_nC5_PR,eq12_C3_PR,eq12_iC4_PR,eq12_nC4_PR
eq12_iC5_PR,eq12_nC5_PR,eq13_PR,eq14_PR
*Raoult Law Properties packages*********************************************************************
eq11_C3_RL,eq11_iC4_RL,eq11_nC4_RL,eq11_iC5_RL,eq11_nC5_RL,eq12_C3_RL,eq12_iC4_RL,eq12_nC4_RL
eq12_iC5_RL,eq12_nC5_RL,eq13_RL,eq14_RL
****************************************************************************************************
*Objective function*********************************************************************************
eq15
****************************************************************************************************
;
positive variable
T(s)       stages temperatures [K]
x(s,i)     stages liquid phase molar composition
y(s,i)     stages vapor phase molar composition
D          top product molar flow [gmol(sec-1)]
Qreb       reboiler heat duty [kW]
Qcond      condenser heat duty [kW]
fugl(s,i)  liquid phase component fugacity [bar]
fugv(s,i)  vapor phase component fugacity [bar]
L(s)       stages liquid phase molar flow [gmol(sec-1)]
V(s)       stages vapor phase molar flow [gmol(sec-1)]
LF(s)      stages feed inlet molar flow [gmol(sec-1)]
;
variables
Hv(s)      stage liquid phase enthalpy [kJ(gmol-1)]
Hl(s)      stage vapor phase enthalpy  [kJ(gmol-1)]
Fobj       Objetive function
;

***Condenser****************************************************************************************
eq1(s,i)$condenser(s).. V(s+1)*y(s+1,i) =e= (L(s)  + D)*x(s,i);
eq2(s)$condenser(s)..   V(s+1)*Hv(s+1)  =e= L(s)*Hl(s)  + D*Hl(s) + Qcond;
****************************************************************************************************

***Reboiler*****************************************************************************************
eq3(s,i)$reboiler(s).. L(s-1)*x(s-1,i)       =e= V(s)*y(s,i) + L(s)*x(s,i);
eq4(s)$reboiler(s)..   L(s-1)*Hl(s-1) + Qreb =e= V(s)*Hv(s)  + L(s)*Hl(s);
****************************************************************************************************

***trays*******************************************************************************************
eq5(s,i)$trays(s).. LF(s)*xf(i) + L(s-1)*x(s-1,i) + V(s+1)*y(s+1,i)   =e= V(s)*y(s,i) + L(s)*x(s,i);
eq6(s)$trays(s)..   LF(s)*Hlf   + L(s-1)*Hl(s-1)  + V(s+1)*Hv(s+1)    =e= V(s)*Hv(s)  + L(s)*Hl(s);
****************************************************************************************************

*Feed location (NLP approach)***********************************************************************
eq7..      sum(s$trays(s),LF(s)) =e= LFF;
****************************************************************************************************

*Mole fraction**************************************************************************************
eq8(s)..   sum(i,x(s,i)) =e= 1;
eq9(s)..   sum(i,y(s,i)) =e= 1;
****************************************************************************************************

*euqilibrium equation********************************************************************************
eq10(s,i).. fugv(s,i) =e= fugl(s,i);
****************************************************************************************************

*PengRobinson***************************************************************************************
eq11_C3_PR(s)..   fugl(s,'C3')  =e= f1_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq11_iC4_PR(s)..  fugl(s,'iC4') =e= f2_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq11_nC4_PR(s)..  fugl(s,'nC4') =e= f3_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq11_iC5_PR(s)..  fugl(s,'iC5') =e= f4_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq11_nC5_PR(s)..  fugl(s,'nC5') =e= f5_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq12_C3_PR(s)..   fugv(s,'C3')  =e= f1_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
eq12_iC4_PR(s)..  fugv(s,'iC4') =e= f2_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
eq12_nC4_PR(s)..  fugv(s,'nC4') =e= f3_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
eq12_iC5_PR(s)..  fugv(s,'iC5') =e= f4_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
eq12_nC5_PR(s)..  fugv(s,'nC5') =e= f5_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
eq13_PR(s)..      Hl(s)         =e= 1e-3*h_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq14_PR(s)..      Hv(s)         =e= 1e-3*h_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
****************************************************************************************************

*Raoult Law*****************************************************************************************
eq11_C3_RL(s)..   fugl(s,'C3')  =e= f1r_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq11_iC4_RL(s)..  fugl(s,'iC4') =e= f2r_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq11_nC4_RL(s)..  fugl(s,'nC4') =e= f3r_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq11_iC5_RL(s)..  fugl(s,'iC5') =e= f4r_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq11_nC5_RL(s)..  fugl(s,'nC5') =e= f5r_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq12_C3_RL(s)..   fugv(s,'C3')  =e= f1r_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
eq12_iC4_RL(s)..  fugv(s,'iC4') =e= f2r_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
eq12_nC4_RL(s)..  fugv(s,'nC4') =e= f3r_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
eq12_iC5_RL(s)..  fugv(s,'iC5') =e= f4r_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
eq12_nC5_RL(s)..  fugv(s,'nC5') =e= f5r_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
eq13_RL(s)..      Hl(s)         =e= 1e-3*hr_liq(T(s),P,x(s,'C3'),x(s,'iC4'),x(s,'nC4'),x(s,'iC5'),x(s,'nC5'));
eq14_RL(s)..      Hv(s)         =e= 1e-3*hr_vap(T(s),P,y(s,'C3'),y(s,'iC4'),y(s,'nC4'),y(s,'iC5'),y(s,'nC5'));
****************************************************************************************************

*Objetive Function**********************************************************************************
eq15.. Fobj =e= 0.2*Qcond  + Qreb;
****************************************************************************************************

model MESH  /eq1,eq2,eq3,eq4,eq5,eq6,eq7,eq8,eq9,eq10/;
model PengR /eq11_C3_PR,eq11_iC4_PR,eq11_nC4_PR,eq11_iC5_PR,eq11_nC5_PR,eq12_C3_PR,eq12_iC4_PR,eq12_nC4_PR
eq12_iC5_PR,eq12_nC5_PR,eq13_PR,eq14_PR/;
model RLaw  /eq11_C3_RL,eq11_iC4_RL,eq11_nC4_RL,eq11_iC5_RL,eq11_nC5_RL,eq12_C3_RL,eq12_iC4_RL,eq12_nC4_RL
eq12_iC5_RL,eq12_nC5_RL,eq13_RL,eq14_RL/;

model idealcollum     /MESH,RLaw,eq15/;
model rigourouscollum /MESH,PengR,eq15/;

*Variable Bound*************************************************************************************
x.lo(s,i)=0;
y.lo(s,i)=0;
x.up(s,i)=1;
y.up(s,i)=1;

*Key component
x.up('s11','nC4')= LKB;
x.up('s1','iC5') = HKT;
****************************************************************************************************

*Initialization*************************************************************************************
T.l(s)=370;

x.l(s,i)=0.2;
y.l(s,i)=0.2;

L.l(s)= 10;
V.l(s)= 10;
D.l   = 90;
LF.l(s) = LFF/9;

Qreb.l=300;
Qcond.l=300;

fugl.l(s,'C3')  = f1r_liq(T.l(s),P,x.l(s,'C3'),x.l(s,'iC4'),x.l(s,'nC4'),x.l(s,'iC5'),x.l(s,'nC5'));
fugl.l(s,'iC4') = f2r_liq(T.l(s),P,x.l(s,'C3'),x.l(s,'iC4'),x.l(s,'nC4'),x.l(s,'iC5'),x.l(s,'nC5'));
fugl.l(s,'nC4') = f3r_liq(T.l(s),P,x.l(s,'C3'),x.l(s,'iC4'),x.l(s,'nC4'),x.l(s,'iC5'),x.l(s,'nC5'));
fugl.l(s,'iC5') = f4r_liq(T.l(s),P,x.l(s,'C3'),x.l(s,'iC4'),x.l(s,'nC4'),x.l(s,'iC5'),x.l(s,'nC5'));
fugl.l(s,'nC5') = f5r_liq(T.l(s),P,x.l(s,'C3'),x.l(s,'iC4'),x.l(s,'nC4'),x.l(s,'iC5'),x.l(s,'nC5'));
fugv.l(s,'C3')  = f1r_vap(T.l(s),P,y.l(s,'C3'),y.l(s,'iC4'),y.l(s,'nC4'),y.l(s,'iC5'),y.l(s,'nC5'));
fugv.l(s,'iC4') = f2r_vap(T.l(s),P,y.l(s,'C3'),y.l(s,'iC4'),y.l(s,'nC4'),y.l(s,'iC5'),y.l(s,'nC5'));
fugv.l(s,'nC4') = f3r_vap(T.l(s),P,y.l(s,'C3'),y.l(s,'iC4'),y.l(s,'nC4'),y.l(s,'iC5'),y.l(s,'nC5'));
fugv.l(s,'iC5') = f4r_vap(T.l(s),P,y.l(s,'C3'),y.l(s,'iC4'),y.l(s,'nC4'),y.l(s,'iC5'),y.l(s,'nC5'));
fugv.l(s,'nC5') = f5r_vap(T.l(s),P,y.l(s,'C3'),y.l(s,'iC4'),y.l(s,'nC4'),y.l(s,'iC5'),y.l(s,'nC5'));
Hl.l(s)         = 1e-3*hr_liq(T.l(s),P,x.l(s,'C3'),x.l(s,'iC4'),x.l(s,'nC4'),x.l(s,'iC5'),x.l(s,'nC5'));
Hv.l(s)         = 1e-3*hr_vap(T.l(s),P,y.l(s,'C3'),y.l(s,'iC4'),y.l(s,'nC4'),y.l(s,'iC5'),y.l(s,'nC5'));
****************************************************************************************************


$ontext
*For simulation purpose*****************************************************************************
*Collum specification
L.fx('s1')= 15.896154000;
*Bottom product flowe rate
L.fx('s11')=6.4387;
*Feed on stage 6
LF.fx(s)=0;
LF.fx('s6')=LFF;
****************************************************************************************************
$offtext


option nlp=conopt;
*Solve using Raoult Law (ideal)*********************************************************************
solve idealcollum using nlp minimizing Fobj;
*Solve using Peng Robinson**************************************************************************
rigourouscollum.holdfixed=1;
solve rigourouscollum using nlp minimizing Fobj;
****************************************************************************************************


